@@ -13,16 +13,16 @@ module Benchmarks.Stream
 
 import Control.DeepSeq (NFData (..))
 import Criterion (Benchmark, bgroup, bench, nf)
-import Data.Text.Fusion.Internal (Step (..), Stream (..))
+import Data.Text.Internal.Fusion.Types (Step (..), Stream (..))
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.Encoding.Error as E
-import qualified Data.Text.Encoding.Fusion as T
-import qualified Data.Text.Encoding.Fusion.Common as F
-import qualified Data.Text.Fusion as T
+import qualified Data.Text.Internal.Encoding.Fusion as T
+import qualified Data.Text.Internal.Encoding.Fusion.Common as F
+import qualified Data.Text.Internal.Fusion as T
 import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy.Encoding as TL
-import qualified Data.Text.Lazy.Encoding.Fusion as TL
-import qualified Data.Text.Lazy.Fusion as TL
+import qualified Data.Text.Internal.Lazy.Encoding.Fusion as TL
+import qualified Data.Text.Internal.Lazy.Fusion as TL
 import qualified Data.Text.Lazy.IO as TL
 
 instance NFData a => NFData (Stream a) where
